/* geist_imlib.h

Copyright (C) 1999,2000 Tom Gilbert.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <Imlib2.h>
#include <stdarg.h>
#include "geist.h"

#ifndef GEIST_IMLIB_H
#define GEIST_IMLIB_H

#define CLIP(x, y, w, h, xx, yy, ww, hh) \
{ \
   if ((yy) > y) { h -= (yy) - y; y = (yy); } \
   if ((yy) + hh < y + h) { h -= y + h - ((yy) + (hh)); } \
   if ((xx) > x) { w -= (xx) - x; x = (xx); } \
   if ((xx) + (ww) < x + w) { w -= x + w - ((xx) + (ww)); } \
}

int geist_imlib_load_image(Imlib_Image * im, char *filename);
int geist_imlib_image_get_width(Imlib_Image im);
int geist_imlib_image_get_height(Imlib_Image im);
int geist_imlib_image_has_alpha(Imlib_Image im);
const char *geist_imlib_image_get_filename(Imlib_Image im);
void geist_imlib_free_image_and_decache(Imlib_Image im);
void geist_imlib_render_image_on_drawable(Drawable d, Imlib_Image im, int x,
                                          int y, char dither, char blend,

                                          char alias);
void geist_imlib_render_image_on_drawable_with_rotation(Drawable d,
                                                        Imlib_Image im, int x,
                                                        int y, double angle,

                                                        char dither,
                                                        char blend,
                                                        char alias);
void geist_imlib_render_image_part_on_drawable_at_size(Drawable d,
                                                       Imlib_Image im, int sx,
                                                       int sy, int sw, int sh,
                                                       int dx, int dy, int dw,
                                                       int dh, char dither,

                                                       char blend,
                                                       char alias);
void geist_imlib_render_image_part_on_drawable_at_size_with_rotation(Drawable
                                                                     d,

                                                                     Imlib_Image
                                                                     im,
                                                                     int sx,
                                                                     int sy,
                                                                     int sw,
                                                                     int sh,
                                                                     int dx,
                                                                     int dy,
                                                                     int dw,
                                                                     int dh,
                                                                     double
                                                                     angle,
                                                                     char
                                                                     dither,
                                                                     char
                                                                     blend,
                                                                     char
                                                                     alias);
void geist_imlib_image_fill_rectangle(Imlib_Image im, int x, int y, int w,
                                      int h, int r, int g, int b, int a);
void geist_imlib_text_draw(Imlib_Image im, Imlib_Font fn, geist_style *s,int x, int y,
                           char *text, Imlib_Text_Direction dir, int r, int g,
                           int b, int a);
void geist_imlib_get_text_size(Imlib_Font fn, char *text, geist_style *s, int *w, int *h,
                               Imlib_Text_Direction dir);
Imlib_Image geist_imlib_clone_image(Imlib_Image im);
char *geist_imlib_image_format(Imlib_Image im);
char **geist_imlib_list_fonts(int *num);
void geist_imlib_render_image_on_drawable_at_size(Drawable d, Imlib_Image im,
                                                  int x, int y, int w, int h,
                                                  char dither, char blend,

                                                  char alias);
void geist_imlib_render_image_on_drawable_at_size_with_rotation(Drawable d,
                                                                Imlib_Image
                                                                im, int x,
                                                                int y, int w,

                                                                int h,
                                                                double angle,
                                                                char dither,
                                                                char blend,
                                                                char alias);
void geist_imlib_blend_image_onto_image(Imlib_Image dest_image,
                                        Imlib_Image source_image,
                                        char merge_alpha, int sx, int sy,
                                        int sw, int sh, int dx, int dy,
                                        int dw, int dh, char dither,
                                        char blend, char alias);
void geist_imlib_blend_image_onto_image_with_rotation(Imlib_Image dest_image,
                                                      Imlib_Image
                                                      source_image,
                                                      char merge_alpha,
                                                      int sx, int sy, int sw,
                                                      int sh, int dx, int dy,
                                                      int dw, int dh,
                                                      double angle,

                                                      char dither, char blend,
                                                      char alias);
Imlib_Image geist_imlib_create_cropped_scaled_image(Imlib_Image im, int sx,
                                                    int sy, int sw, int sh,
                                                    int dw, int dh,

                                                    char alias);
void geist_imlib_apply_color_modifier_to_rectangle(Imlib_Image im, int x,
                                                   int y, int w, int h,
                                                   DATA8 * rtab, DATA8 * gtab,
                                                   DATA8 * btab,

                                                   DATA8 * atab);
void geist_imlib_image_set_has_alpha(Imlib_Image im, int alpha);
void geist_imlib_save_image(Imlib_Image im, char *file);
void geist_imlib_free_font(Imlib_Font fn);
void geist_imlib_free_image(Imlib_Image im);
void geist_imlib_image_draw_line(Imlib_Image im, int x1, int y1, int x2,
                                 int y2, char make_updates, int r, int g,
                                 int b, int a);
void geist_imlib_image_set_has_alpha(Imlib_Image im, int alpha);
void geist_image_save_image(Imlib_Image im, char *file);
void geist_imlib_free_font(Imlib_Font fn);
Imlib_Image geist_imlib_create_rotated_image(Imlib_Image im, double angle);
void geist_imlib_image_tile(Imlib_Image im);
void geist_imlib_image_blur(Imlib_Image im, int radius);
void geist_imlib_image_sharpen(Imlib_Image im, int radius);
void geist_imlib_image_draw_rectangle(Imlib_Image im, int x, int y, int w,
                                      int h, int r, int g, int b, int a);
DATA8 geist_imlib_image_part_is_transparent(Imlib_Image im, int x, int y);
void geist_imlib_line_clip_and_draw(Imlib_Image dest, int x0, int y0, int x1,
                               int y1, int cx, int cy, int cw, int ch,
                               int r, int g, int b, int a);
void geist_imlib_image_fill_polygon(Imlib_Image im, ImlibPolygon poly, int r,
                                    int g, int b, int a, unsigned char alias,
                                    int cx, int cy, int cw, int ch);
void geist_imlib_image_draw_polygon(Imlib_Image im, ImlibPolygon poly, int r,
                                    int g, int b, int a, unsigned char closed,
                                    unsigned char alias, int cx, int cy,
                                    int cw, int ch);

#endif
